#include <Servo.h>
 
const byte SERVO_A_ANGLE_MIN = 0;
const byte SERVO_A_ANGLE_MAX = 1;
const byte SERVO_EN_MOUVEMENT_VERS_ANGLE_MAX = 2;
const byte SERVO_EN_MOUVEMENT_VERS_ANGLE_MIN = 3;
 
const int angleMin = 1250;
const int angleMax = 1750;

/*
 * Descripteur d'un servomoteur
 */
class DescripteurServo {
    Servo objetServo; /* Objet issu de la bibliothèque Servo de l'Arduino */
    int vitesse;      /* La vitesse de déplacement du servomoteur */
    int angle;        /* L'angle du servo-moteur */
    int pin;          /* La broche sur laquelle il est connecté */ 
    byte etatServo;   /* son état : SERVO_A_ANGLE_MIN, SERVO_A_ANGLE_MAX
                         SERVO_EN_MOUVEMENT_VERS_ANGLE_MAX ou
                         SERVO_EN_MOUVEMENT_VERS_ANGLE_MIN    */

  public:
    /* Constructeur, le servo est mis arreté à l'angle min */
    DescripteurServo()
    {
        angle = angleMin;
        vitesse = 0;
        etatServo = SERVO_A_ANGLE_MIN;
    }
    
    /* Connexion, le servo est connecté à une broche et asservi */
    void connecte(int pinDeConnection)
    {
        pin = pinDeConnection;
        objetServo.attach(pinDeConnection);
    }
  
    /*
     * Ajoute la vitesse à l'angle. Stoppe le servo quand l'angle min ou max
     * est atteint. Stoppe son asservissement
     * Doit etre appelé péridioquement à un intervalle de temps fixe
     */
    void gereServo()
    {
        objetServo.writeMicroseconds(angle);
     
        angle += vitesse;
     
        if (angle > angleMax) {
            angle = angleMax;
            vitesse = 0;
            objetServo.detach();
            etatServo = SERVO_A_ANGLE_MAX;
        }
        else if (angle < angleMin) {
          angle = angleMin;
          vitesse = 0;
          objetServo.detach();
          etatServo = SERVO_A_ANGLE_MIN;
        }
    }
    
    /*
     * Indique au servo qu'il doit gagner la position opposée à
     * la position actuelle
     */
    void evenementServo()
    {
        switch (etatServo) {
          case SERVO_A_ANGLE_MIN:
            objetServo.attach(pin);
          case SERVO_EN_MOUVEMENT_VERS_ANGLE_MIN:
            vitesse =  1;
            etatServo = SERVO_EN_MOUVEMENT_VERS_ANGLE_MAX;
            break;
          case SERVO_A_ANGLE_MAX:
            objetServo.attach(pin);
          case SERVO_EN_MOUVEMENT_VERS_ANGLE_MAX:
            vitesse = -1;
            etatServo = SERVO_EN_MOUVEMENT_VERS_ANGLE_MIN;
            break;
        } 
    }
};
 
DescripteurServo servoMoteur[8];
 
const byte NON_PRESSE = 0;
const byte ENFONCE = 1;
const byte PRESSE = 2;
 
byte etatAutomate = NON_PRESSE;
int etatPoussoir = -1;
 
const byte AUCUN_EVENEMENT = 0;
const byte EVENEMENT_PRESSE = 1;
const byte EVENEMENT_RELACHE = 2;
 
const int pinPoussoirs = 0;
 
int lirePoussoirs()
{
    int resultat;
    int numPoussoir = (analogRead(pinPoussoirs) + 64) / 128;
    
    int nouvelEtatPoussoir = etatPoussoir; /* à priori rien ne change */
 
    switch (etatAutomate) {
        case NON_PRESSE:
            if (numPoussoir < 8)
                etatAutomate = ENFONCE;
            break;
        case ENFONCE:
            if (numPoussoir < 8) {
                etatAutomate = PRESSE;
                nouvelEtatPoussoir = numPoussoir;
            }
            else {
                etatAutomate = NON_PRESSE;
            }
            break;
        case PRESSE:
            if (numPoussoir == 8) {
                etatAutomate = NON_PRESSE;
                nouvelEtatPoussoir = -1;
            }
            break;
    }
    
    return nouvelEtatPoussoir;
}
 
/*
 * construction d'un événement en comparant
 * le nouvel état des poussoirs avec l'état précédent.
 */
byte lireEvenement(int *numPoussoir)
{
    byte evenement;
    int nouvelEtatPoussoir = lirePoussoirs();
    
    if (nouvelEtatPoussoir == etatPoussoir)
        evenement = AUCUN_EVENEMENT;
    if (nouvelEtatPoussoir >= 0 && etatPoussoir == -1) 
        evenement = EVENEMENT_PRESSE;
    if (nouvelEtatPoussoir == -1 && etatPoussoir >= 0) 
        evenement = EVENEMENT_RELACHE;
 
    etatPoussoir = nouvelEtatPoussoir;
    *numPoussoir = etatPoussoir;
    
    return evenement;
}
 
void setup()
{
    /* Initialisation des servos */
    int numServo;
  
    for (numServo = 0; numServo < 8; numServo++)
        servoMoteur[numServo].connecte(numServo + 2);
}

 
void loop()
{
    int numServo;
  
    for (numServo = 0; numServo < 8; numServo++) servoMoteur[numServo].gereServo();
 
    byte evenement = lireEvenement(&numServo);
  
    if (evenement == EVENEMENT_PRESSE) {
        servoMoteur[numServo].evenementServo();
    }
  
    delay(3);
}
